# Copyright (C) 2009  Stephane Galland <galland@arakhne.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

=pod

=head1 NAME

GPrepareGalleryWindow - A GTK user interface for preparegallery

=head1 DESCRIPTION

GPrepareGalleryWindow is a Perl module, which permits to
display a Gtk user interface for prepare-gallery.

=head1 SYNOPSYS

GPrepareGalleryWindow->new($) ;

=head1 METHOD DESCRIPTIONS

This section contains only the methods in GPrepareGalleryWindow.pm itself.

=over

=cut

package GPrepareGalleryWindow;

our @ISA = qw( Gtk2::Window );
our @EXPORT = qw();
our @EXPORT_OK = qw();

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
use Exporter;

use File::Spec;
use File::Basename;
use Glib qw(TRUE FALSE);
use Gtk2 qw/-init -threads-init/;

use Locale::gettext;
use POSIX; # Needed for setlocale()

use Version;

use constant DEFAULT_WIDTH => 600;
use constant DEFAULT_HEIGHT => 450;

#------------------------------------------------------
#
# Global vars
#
#------------------------------------------------------

# Version number
my $VERSION = "2.1" ;

BEGIN{
	setlocale(LC_ALL, "");
}

#------------------------------------------------------
#
# Functions
#
#------------------------------------------------------

my $QUITGTK = FALSE;

my $ERROR_MSG = undef;

sub gtkDie($) {
	$ERROR_MSG = $_[0] || 'Error';
	$QUITGTK = TRUE;
}

=pod

=item * quitGtk()

Quit the Gtk main loop.

=cut
sub quitGtk() {
	$QUITGTK = TRUE;
}

=pod

=item * mustQuitGtk()

Indicates of Quit the Gtk main loop.

=cut
sub mustQuitGtk() {
	return $QUITGTK;
}

=pod

=item * resetQuitGtkFlag()

Reset the flag that indicates if the Gtk main loop must be quitted.

=cut
sub resetQuitGtkFlag() {
	$QUITGTK = FALSE;
}


sub onQuit(@) {
	quitGtk();
}

sub onGenerate(@) {
	my $self = shift;

	if ($self->{'GENERATION_CALLBACK'}->(
		$self->{'WIDGET'}{'target'}->get_filename(),
		$self->{'WIDGET'}{'quality'}->get_value(),
		$self->{'WIDGET'}{'width'}->get_value(),
		$self->{'WIDGET'}{'height'}->get_value(),
		$self->{'WIDGET'}{'name'}->get_text(),
		$self->{'WIDGET'}{'firstnumber'}->get_value(),
		$self->{'WIDGET'}{'title'}->get_text(),
		$self->{'WIDGET'}{'description'}->get_text(),
		$self->{'WIDGET'}{'textonly'}->get_active())) {
		quitGtk();
	}
}

sub onLockUnlock(@) {
	my $self = shift;

	if ($self->{'locked'}) {
		$self->{'locked'} = FALSE;
		$self->{'WIDGET'}{'lockunlock'}->set_image($self->getLockIcon());
		$self->{'WIDGET'}{'lockunlock'}->set_tooltip_text($self->getLockTooltip());
	}
	else {
		$self->{'locked'} = TRUE;
		$self->{'WIDGET'}{'lockunlock'}->set_image($self->getLockIcon());
		$self->{'WIDGET'}{'lockunlock'}->set_tooltip_text($self->getLockTooltip());
		my $width = $self->{'WIDGET'}{'width'}->get_value();
		my $height = $self->{'WIDGET'}{'height'}->get_value();
		$self->{'lock_ratio'} = $height / $width;
	}
}

sub onWidthChanged(@) {
	my $self = shift;
	my $ctrl = shift;

	if ($self->{'locked'} && !$self->{'lock_resource'}) {

		$self->{'lock_resource'} = TRUE;

		my $ratio = $self->{'lock_ratio'};
		my $width = $ctrl->get_value();

		my $newHeight = $self->{'lock_ratio'} * $width;
		if ($newHeight<1) {
			$newHeight = 1;
		}

		$self->{'WIDGET'}{'height'}->set_value($newHeight);

		$self->{'lock_resource'} = FALSE;
	}
}

sub onHeightChanged(@) {
	my $self = shift;
	my $ctrl = shift;

	if ($self->{'locked'} && !$self->{'lock_resource'}) {

		$self->{'lock_resource'} = TRUE;

		my $ratio = $self->{'lock_ratio'};
		my $height = $ctrl->get_value();

		my $newWidth = $height / $self->{'lock_ratio'};
		if ($newWidth<10) {
			$newWidth = 10;
		}

		$self->{'WIDGET'}{'width'}->set_value($newWidth);

		$self->{'lock_resource'} = FALSE;
	}
}

sub getCurrentLocale() {
	my $lang = $ENV{'LC_ALL'} || $ENV{'LANG'};
	if ($lang =~ /^([a-zA-Z]+_[a-zA-Z]+)/) {
		return "$1";
	}
	elsif ($lang =~ /^([a-zA-Z]+)/) {
		return "$1";
	}
	return undef;
}

=pod

=item * new($)

Contructor.

Parameters are:

=over 4

=item icon for the dialog box.

=item subroutine to invoke when generation button was clicked.

=back

=cut
sub new($$$) : method {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $icon = shift;
	my $path = shift;
	my $generationCallback = shift;

	my $self = bless($class->SUPER::new('toplevel'), $class) ;

	# Locale
	$self->{'current_locale'} = getCurrentLocale();

	$self->{'locale'} = Locale::gettext->domain_raw('GPrepareGalleryWindow');
	$self->{'locale'}->codeset($self->{'current_locale'});
	$self->{'locale'}->dir(File::Spec->catfile("$path",'po'));
	$self->{'locale'}->codeset($self->{'current_locale'});

	# Rest of initialization
	$self->{'locked'} = TRUE;
	$self->{'lock_picture'} = File::Spec->catfile("$path", 'lock.png');
	$self->{'unlock_picture'} = File::Spec->catfile("$path", 'unlock.png');
	$self->{'lock_ratio'} = DEFAULT_HEIGHT / DEFAULT_WIDTH;

	$self->{'GENERATION_CALLBACK'} = $generationCallback;

	# Set the windows attributes
	$self->set_title ($self->T("Prepare a Photo Gallery v{}", preparegallery_version()));
	if ($icon) {
		$self->set_icon_from_file($icon);
	}
	$self->connectSignal($self,'delete-event','onQuit');

	return $self;
}

sub T($@) {
	my $self = shift;
	my $txt = shift || '';
	$txt = $self->{'locale'}->get($txt);
	if (@_) {
		for(my $i=0; $i<@_; $i++) {
			$txt =~ s/[\$\#]\Q{$i}\E/$_[$i]/g;
			$txt =~ s/[\$\#]\Q$i\E/$_[$i]/g;
		}
		my $i=0;
		$txt =~ s/\Q{}\E/my $v;if ($i<@_) {$v=$_[$i]||'';$i++;} else {$v="{}";}"$v";/eg;
	}
	return $txt;
}

sub getLockIconPath() {
	my $self = shift;

	if ($self->{'locked'}) {
		return $self->{'lock_picture'};
	}
	else {
		return $self->{'unlock_picture'};
	}
}

sub getLockIcon() {
	my $self = shift;
	my $path = $self->getLockIconPath();
	return Gtk2::Image->new_from_file("$path");
}

sub getLockTooltip() {
	my $self = shift;

	if ($self->{'locked'}) {
		return $self->T('Preserve aspect ratio');
	}
	else {
		return $self->T('Do not preserve aspect ratio');
	}
}

=pod

=item * connectSignal($$$)

Initializing the dialog content before displaying.

=over 4

=item the Gtk object to connect to the signal

=item the name of the signal to connect

=item the name of the subroutine to call when the signal was triggered.

=back

=cut

sub connectSignal($$$) : method {
	my $self = shift;
	my $obj = shift;
	my $signame = shift;
	my $procname = shift;

	my $proc = sub {
		my $refself = $self;
		my $refprocname = $procname;
		my $refobj = $obj;
		eval("\$refself->$procname(\@_);");
	};

	$obj->signal_connect ( $signame => $proc );
	return 1;
}

=pod

=item * showDialog()

Show the dialog.

=cut
sub showDialog() : method {
	my $self = shift;
	$self->initializeDialogContent();
	$self->doLoop();
	$self->destroyDialogContent();
}

=pod

=item * initializeDialogContent()

Initializing the dialog content before displaying.

=cut
sub initializeDialogContent() : method {
	my $self = shift;

	#$self->initLocale('gpreparegallery');

	# Main layout
	my $mainLayout = Gtk2::Table->new (
						2, #rows
						1, #columns
						FALSE); #non uniform
	$self->add ($mainLayout);
	
	# Initialize the main panel
	my $mainPanel = Gtk2::Table->new (
						10, #rows
						3, #columns
						FALSE); #non uniform
	$mainLayout->attach ($mainPanel, 
				0,1,0,1, # left, right, top and bottom columns
				['fill','expand'],['fill','expand'], # x and y options
				5,5); # horizontal and vertical paddings

	my $currentRow = 0;

	# Source directory
	{
		my $label = Gtk2::Label->new ($self->T("Directory where photos are located:"));
		$mainPanel->attach ($label, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$self->{'WIDGET'}{'target'} = Gtk2::FileChooserButton->new ($self->T("Select the directory of the photos"), 'select-folder');
		$mainPanel->attach ($self->{'WIDGET'}{'target'},
					1,2,$currentRow,$currentRow+1, # left, right, top and bottom columns
					['expand','fill'],['expand','fill'], # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Photo width
	{
		my $label = Gtk2::Label->new ($self->T("Width of the photos (pixels):"));
		$mainPanel->attach ($label, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$self->{'WIDGET'}{'width'} = Gtk2::HScale->new_with_range (10,800,1);
		$self->{'WIDGET'}{'width'}->set_value(DEFAULT_WIDTH);
		$self->connectSignal($self->{'WIDGET'}{'width'},'value-changed','onWidthChanged');
		$mainPanel->attach ($self->{'WIDGET'}{'width'}, 
					1,2,$currentRow,$currentRow+1, # left, right, top and bottom columns
					['expand','fill'],['expand','fill'], # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Size lock/unlock
	{
		$self->{'WIDGET'}{'lockunlock'} = Gtk2::Button->new();
		$self->{'WIDGET'}{'lockunlock'}->set_image($self->getLockIcon());
		$self->{'WIDGET'}{'lockunlock'}->set_tooltip_text($self->getLockTooltip());
		$self->connectSignal($self->{'WIDGET'}{'lockunlock'},'clicked','onLockUnlock');
		$mainPanel->attach ($self->{'WIDGET'}{'lockunlock'}, 
					2,3,$currentRow,$currentRow+1, # left, right, top and bottom columns
					['expand','fill'],['expand','fill'], # x and y options
					0,0); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Photo height
	{
		my $label = Gtk2::Label->new ($self->T("Height of the photos (pixels):"));
		$mainPanel->attach ($label, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$self->{'WIDGET'}{'height'} = Gtk2::HScale->new_with_range (1,800,1);
		$self->{'WIDGET'}{'height'}->set_value(DEFAULT_HEIGHT);
		$self->connectSignal($self->{'WIDGET'}{'height'},'value-changed','onHeightChanged');
		$mainPanel->attach ($self->{'WIDGET'}{'height'}, 
					1,2,$currentRow,$currentRow+1, # left, right, top and bottom columns
					['expand','fill'],['expand','fill'], # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Name of the gallery
	{
		my $label = Gtk2::Label->new ($self->T("Name of the gallery:"));
		$mainPanel->attach ($label, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$self->{'WIDGET'}{'name'} = Gtk2::Entry->new ();
		$self->{'WIDGET'}{'name'}->set_text("gallery");
		$mainPanel->attach ($self->{'WIDGET'}{'name'}, 
					1,2,$currentRow,$currentRow+1, # left, right, top and bottom columns
					['expand','fill'],['expand','fill'], # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# First photo number
	{
		my $label = Gtk2::Label->new ($self->T("First photo number:"));
		$mainPanel->attach ($label, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$self->{'WIDGET'}{'firstnumber'} = Gtk2::HScale->new_with_range (1,1000,1);
		$self->{'WIDGET'}{'firstnumber'}->set_value(1);
		$mainPanel->attach ($self->{'WIDGET'}{'firstnumber'}, 
					1,2,$currentRow,$currentRow+1, # left, right, top and bottom columns
					['expand','fill'],['expand','fill'], # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Title
	{
		my $label = Gtk2::Label->new ($self->T("Title for the photos:"));
		$mainPanel->attach ($label, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$self->{'WIDGET'}{'title'} = Gtk2::Entry->new ();
		$self->{'WIDGET'}{'title'}->set_text("");
		$mainPanel->attach ($self->{'WIDGET'}{'title'}, 
					1,2,$currentRow,$currentRow+1, # left, right, top and bottom columns
					['expand','fill'],['expand','fill'], # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Description
	{
		my $label = Gtk2::Label->new ($self->T("Description of the photos:"));
		$mainPanel->attach ($label, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$self->{'WIDGET'}{'description'} = Gtk2::Entry->new ();
		$self->{'WIDGET'}{'description'}->set_text("");
		$mainPanel->attach ($self->{'WIDGET'}{'description'}, 
					1,2,$currentRow,$currentRow+1, # left, right, top and bottom columns
					['expand','fill'],['expand','fill'], # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Photo quality
	{
		my $label = Gtk2::Label->new ($self->T("Quality of the photos (%):"));
		$mainPanel->attach ($label, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$self->{'WIDGET'}{'quality'} = Gtk2::HScale->new_with_range (0,100,1);
		$self->{'WIDGET'}{'quality'}->set_value(100);
		$mainPanel->attach ($self->{'WIDGET'}{'quality'}, 
					1,2,$currentRow,$currentRow+1, # left, right, top and bottom columns
					['expand','fill'],['expand','fill'], # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Only text files
	{
		$self->{'WIDGET'}{'textonly'} = Gtk2::CheckButton->new ($self->T("Create only the text files"));
		$mainPanel->attach ($self->{'WIDGET'}{'textonly'}, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Progress Bar
	{
		$self->{'WIDGET'}{'progress'} = Gtk2::ProgressBar->new ();
		$self->{'WIDGET'}{'progress'}->set_orientation('left-to-right');
		$self->{'WIDGET'}{'progress'}->set_fraction(0);
		$mainPanel->attach ($self->{'WIDGET'}{'progress'}, 
					0,1,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		my $label = Gtk2::Label->new ($self->T("(c) Stephane GALLAND"));
		$mainPanel->attach ($label, 
					1,2,$currentRow,$currentRow+1, # left, right, top and bottom columns
					'shrink','shrink', # x and y options
					2,2); # horizontal and vertical paddings

		$currentRow ++;
	}

	# Close button
	my $buttonAlignment = Gtk2::HButtonBox->new ();
	$buttonAlignment->set_layout('end');

	my $generateButton = Gtk2::Button->new_from_stock ('gtk-execute');
	$self->connectSignal($generateButton,'clicked','onGenerate');
	$buttonAlignment->add ($generateButton);

	my $closeButton = Gtk2::Button->new_from_stock ('gtk-cancel');
	$self->connectSignal($closeButton,'clicked','onQuit');
	$buttonAlignment->add ($closeButton);

	$mainLayout->attach ($buttonAlignment, 
				0,1,1,2, # left, right, top and bottom columns
				['fill','expand'],'shrink', # x and y options
				5,5); # horizontal and vertical paddings
}

=pod

=item * doLoop()

Do the GUI interaction loop. It means that the dialog
must be displayed now and ready for interaction.

=cut
sub doLoop() : method {
	my $self = shift;
	$self->show_all;
	my $quit = FALSE;
	my $lastmsg = '';
	resetQuitGtkFlag();
	while (!mustQuitGtk()) {
		$self->refreshGTK();
	}
	if ($ERROR_MSG) {
		die("$ERROR_MSG");
	}
}

=pod

=item * destroyDialogContent()

Destroying the dialog content after the application was quitted.

=cut
sub destroyDialogContent() : method {
	my $self = shift;
	foreach my $widget (values %{$self->{'WIDGET'}}) {
		$widget->set_sensitive(FALSE);
	}
	$self->{'WIDGET'}{'progress'}->set_sensitive(TRUE);
	$self->refreshGTK();
}

=pod

=item * getIconPath($)

Replies the complete path to the specified icon.

=cut
sub getIconPath($) : method {
	my $self = shift;
	my $iconname = shift;
	
	my $filename = File::Spec->catfile(dirname(__FILE__),"$iconname");

	return $filename if (-e "$filename");
	return undef;
}

sub setProgress($) {
	my $self = shift;
	$self->{'WIDGET'}{'progress'}->set_fraction($_[0]);
	$self->refreshGTK();
}

sub refreshGTK() {
	my $self = shift;
	while (Gtk2->events_pending()) {
		Gtk2->main_iteration();
	}
}

1;
__END__

=back

=head1 COPYRIGHT

(c) Copyright 2009 Stéphane Galland E<lt>galland@arakhne.orgE<gt>, under GPL.

=head1 AUTHORS

=over

=item *

Conceived and initially developed by Stéphane Galland E<lt>galland@arakhne.orgE<gt>.

=back

=head1 SEE ALSO

L<autolatex>
